@@ -49,7 +49,7 @@ You can mix types with natsorted.  This can get around the new
 'unorderable types' issue with Python 3.
 
     >>> import sys
-    >>> a = [6, 4.5, '7', u'2.5']
+    >>> a = [6, 4.5, '7', u'2.5', 'a']
     >>> if sys.version[0] == '3': # Python 3
     ...     try:
     ...         sorted(a)
@@ -57,14 +57,14 @@ You can mix types with natsorted.  This can get around the new
     ...         print(e)
     ... else: # Python 2
     ...     # This will get the doctest to work properly while illustrating the point
-    ...     if sorted(a) == [4.5, 6, u'2.5', '7']:
+    ...     if sorted(a) == [4.5, 6, u'2.5', '7', 'a']:
     ...         print('unorderable types: str() < float()')
     ...
     unorderable types: str() < float()
     >>> natsorted(a)
-    [{u}'2.5', 4.5, 6, {u}'7']
+    [{u}'2.5', 4.5, 6, {u}'7', {u}'a']
 
-Natsort will recursively descend into lists of lists so you can sort by the sublist contents.
+natsort will recursively descend into lists of lists so you can sort by the sublist contents.
 
     >>> data = [['a1', 'a5'], ['a1', 'a40'], ['a10', 'a1'], ['a2', 'a5']]
     >>> sorted(data)
@@ -74,10 +74,12 @@ Natsort will recursively descend into lists of lists so you can sort by the subl
 
 """
 
-from __future__ import unicode_literals
+from __future__ import print_function, division, unicode_literals, absolute_import
 
 import re
 import sys
+from numbers import Number
+from itertools import islice
 
 from .py23compat import u_format, py23_basestring, py23_range, py23_str, py23_zip
 
@@ -126,7 +128,7 @@ def remove_empty(s):
     return s
 
 
-def _number_finder(s, regex, numconv):
+def _number_finder(s, regex, numconv, py3_safe):
     """Helper to split numbers"""
 
     # Split.  If there are no splits, return now
@@ -142,11 +144,35 @@ def _number_finder(s, regex, numconv):
         except ValueError:
             pass
 
-    return s
+    # If the list begins with a number, lead with an empty string.
+    # This is used to get around the "unorderable types" issue.
+    # The _py3_safe function inserts "" between numbers in the list,
+    # and is used to get around "unorderable types" in complex cases.
+    # It is a separate function that needs to be requested specifically
+    # because it is expensive to call.
+    if not isinstance(s[0], py23_basestring):
+        return _py3_safe([''] + s) if py3_safe else [''] + s
+    else:
+        return _py3_safe(s) if py3_safe else s
+
+
+def _py3_safe(parsed_list):
+    """Insert "" between two numbers."""
+    if len(parsed_list) < 2:
+        return parsed_list
+    else:
+        new_list = [parsed_list[0]]
+        nl_append = new_list.append
+        for before, after in py23_zip(islice(parsed_list, 0, len(parsed_list)-1),
+                                      islice(parsed_list, 1, None)):
+            if isinstance(before, Number) and isinstance(after, Number):
+                nl_append("")
+            nl_append(after)
+        return tuple(new_list)
 
 
 @u_format
-def natsort_key(s, number_type=float, signed=True, exp=True):
+def natsort_key(s, number_type=float, signed=True, exp=True, py3_safe=False):
     """\
     Key to sort strings and numbers naturally, not lexicographically.
     It also has basic support for version numbers.
@@ -197,10 +223,24 @@ def natsort_key(s, number_type=float, signed=True, exp=True):
         >>> natsort_key(('a1', 'a10'))
         (({u}'a', 1.0), ({u}'a', 10.0))
 
+    Strings that lead with a number get an empty string at the front of the tuple.
+    This is designed to get around the "unorderable types" issue.
+
+        >>> natsort_key(('15a', '6'))
+        (({u}'', 15.0, {u}'a'), ({u}'', 6.0))
+
     You can give numbers, too.
 
         >>> natsort_key(10)
-        (10,)
+        ({u}'', 10)
+
+    If you have a case where one of your string has two numbers in a row
+    (only possible with "5+5" or "5-5" and signed=True to my knowledge), you
+    can turn on the "py3_safe" option to try to add a "" between sets of two
+    numbers.
+
+        >>> natsort_key('43h7+3', py3_safe=True)
+        ({u}'', 43.0, {u}'h', 7.0, {u}'', 3.0)
 
     """
 
@@ -209,11 +249,11 @@ def natsort_key(s, number_type=float, signed=True, exp=True):
         if hasattr(s, '__getitem__'):
             return tuple(natsort_key(x) for x in s)
         else:
-            return (s,)
+            return ('', s,)
 
     # Convert to the proper tuple and return
     inp_options = (number_type, signed, exp)
-    args = (s,) + regex_and_num_function_chooser[inp_options]
+    args = (s,) + regex_and_num_function_chooser[inp_options] + (py3_safe,)
     try:
         return tuple(_number_finder(*args))
     except KeyError:
@@ -243,10 +283,28 @@ def natsorted(seq, key=lambda x: x, number_type=float, signed=True, exp=True):
         >>> natsorted(b, key=itemgetter(1))
         [({u}'c', {u}'num2'), ({u}'a', {u}'num3'), ({u}'b', {u}'num5')]
 
+    It tries really hard to not get the "unorderable types" error
+
+        >>> a = [46, '5a5b2', 'af5', '5a5-4']
+        >>> natsorted(a)
+        [{u}'5a5-4', {u}'5a5b2', 46, {u}'af5']
+
     """
-    return sorted(seq, key=lambda x: natsort_key(key(x),
-                                                 number_type=number_type,
-                                                 signed=signed, exp=exp))
+    try:
+        return sorted(seq, key=lambda x: natsort_key(key(x),
+                                                     number_type=number_type,
+                                                     signed=signed, exp=exp))
+    except TypeError as e:
+        # In the event of an unresolved "unorderable types" error
+        # attempt to sort again, being careful to prevent this error.
+        if 'unorderable types' in str(e):
+            return sorted(seq, key=lambda x: natsort_key(key(x),
+                                                         number_type=number_type,
+                                                         signed=signed, exp=exp,
+                                                         py3_safe=True))
+        else:
+            # Re-raise if the problem was not "unorderable types"
+            raise
 
 
 @u_format
@@ -270,14 +328,32 @@ def index_natsorted(seq, key=lambda x: x, number_type=float, signed=True, exp=Tr
         >>> index_natsorted(c, key=itemgetter(1))
         [2, 0, 1]
 
+    It tries really hard to not get the "unorderable types" error
+
+        >>> a = [46, '5a5b2', 'af5', '5a5-4']
+        >>> index_natsorted(a)
+        [3, 1, 0, 2]
+
     """
     from operator import itemgetter
     item1 = itemgetter(1)
     # Pair the index and sequence together, then sort by
     index_seq_pair = [[x, key(y)] for x, y in py23_zip(py23_range(len(seq)), seq)]
-    index_seq_pair.sort(key=lambda x: natsort_key(item1(x), 
-                                                  number_type=number_type,
-                                                  signed=signed, exp=exp))
+    try:
+        index_seq_pair.sort(key=lambda x: natsort_key(item1(x), 
+                                                      number_type=number_type,
+                                                      signed=signed, exp=exp))
+    except TypeError as e:
+        # In the event of an unresolved "unorderable types" error
+        # attempt to sort again, being careful to prevent this error.
+        if 'unorderable types' in str(e):
+            index_seq_pair.sort(key=lambda x: natsort_key(item1(x), 
+                                                          number_type=number_type,
+                                                          signed=signed, exp=exp,
+                                                          py3_safe=True))
+        else:
+            # Re-raise if the problem was not "unorderable types"
+            raise
     return [x[0] for x in index_seq_pair]
 
 
